use crate::prelude::*;

#[derive(Debug, Copy, Clone, PartialEq)]
pub enum TileKind {
    Wall,
    Floor,
}

#[derive(Debug, Clone)]
pub struct Tile {
    kind: TileKind,
}

impl Tile {
    pub fn new(kind: TileKind) -> Self {
        Self {
            kind,
        }
    }

    pub fn render_point(&self, ctx: &mut BTerm, p: Point) {
        self.render(ctx, p.x, p.y);
    }

    pub fn render(&self, ctx: &mut BTerm, x: i32, y: i32) {
        match self.kind {
            TileKind::Floor => ctx.set(x, y, WHITE, BLACK, to_cp437(' ')),
            TileKind::Wall => ctx.set(x, y, WHITE, BLACK, to_cp437('#')),
        }
    }

    pub fn can_enter(&self) -> bool {
        match self.kind {
            TileKind::Floor => true,
            TileKind::Wall => false,
        }
    }
}

pub struct Map {
    pub tiles: Vec<Tile>,
    pub width: i32,
    pub height: i32,
}

impl Map {
    pub fn new(width: Option<i32>, height: Option<i32>) -> Self {
        let actual_width = width.unwrap_or(SCREEN_WIDTH);
        let actual_height = height.unwrap_or(SCREEN_HEIGHT);

        Self {
            tiles: vec![Tile::new(TileKind::Floor); (actual_width * actual_height) as usize],
            width: actual_width,
            height: actual_height,
        }
    }

    pub fn tile_at_point(&self, p: Point) -> &Tile {
        self.tile_at(p.x, p.y)
    }

    pub fn tile_at(&self, x: i32, y: i32) -> &Tile {
        &(self.tiles[((y * self.width) + x) as usize])
    }

    pub fn is_in_bounds_point(&self, p: Point) -> bool {
        self.is_in_bounds(p.x, p.y)
    }

    pub fn is_in_bounds(&self, x: i32, y: i32) -> bool {
        x >= 0 && x < self.width && y >= 0 && y < self.height
    }

    // TODO: this will need a given set of coords to look at?
    pub fn render(&self, ctx: &mut BTerm) {
        for y in 0..SCREEN_HEIGHT {
            for x in 0..SCREEN_WIDTH {
                self.tile_at(x, y).render(ctx, x, y);
            }
        }
    }
}
